<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://dabex.eu/woocommerce-gls/
 * @since             1.0.0
 * @package           Gls
 *
 * @wordpress-plugin
 * Plugin Name:       Dabex - Integrare Curier GLS
 * Plugin URI:        https://dabex.eu/woocommerce-gls/
 * Description:       GLS Parcel Order Generator
 * Version:           1.0.0
 * Author:            Dabex Agency
 * Author URI:        https://dabex.eu
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       gls
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'GLS_SHIPPING', '1.0.0' );

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-gls-activator.php
 */
function activate_gls() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-gls-activator.php';
	Gls_Activator::activate();
	// run the install scripts upon plugin activation
	gls_install();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-gls-deactivator.php
 */
function deactivate_gls() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-gls-deactivator.php';
	Gls_Deactivator::deactivate();
	gls_uninstall();
}

register_activation_hook( __FILE__, 'activate_gls' );
register_deactivation_hook( __FILE__, 'deactivate_gls' );

/**
 * Adds database tables used for storing custom order data
 *
 * @global $wpdb;
 */

function gls_install() {
   	global $wpdb;
  	global $db_name;

	// create the ECPT metabox database table
	if($wpdb->get_var("show tables like '$db_name'") != 'glscarrier')
	{
		$sql = 'CREATE TABLE IF NOT EXISTS `glscarrier` (
		`id` int(10) unsigned NOT NULL auto_increment,
		`order_id` TEXT NOT NULL ,
		`ship_parcel_id` varchar(25) NOT NULL ,
		`pickupdate` varchar(25) NOT NULL ,
		`content` varchar(25) NOT NULL ,
		`clientref` varchar(25) NOT NULL ,
		`codamount` varchar(25) NOT NULL ,
		`codref` varchar(25) NOT NULL ,
		`services` text COLLATE utf8_bin NOT NULL,
		`date_add` datetime NOT NULL,
		`date_upd` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
		`cont_id` varchar(25) NOT NULL,
		PRIMARY KEY ( `id` )
		)ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;';

		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta($sql);
	}
}

/**
 * Remove database tables used for storing custom order data
 *
 * @global $wpdb;
 */

function gls_uninstall() {
	global $wpdb;
	global $db_name;
	// drop table;
	$sql = "DROP TABLE IF EXISTS `glscarrier`;";
	$wpdb->query($sql);
	// remove options stored in wp_options
  delete_option( 'gls_cont_1' );
  delete_option( 'gls_cont_2' );
  delete_option( 'gls_cont_3' );
}


/**
 * create extra box in order details
 */
add_action( 'add_meta_boxes', 'gls_tracking_box' );
function gls_tracking_box() {
    add_meta_box(
        'gls-tracking-modal',
        'Genereaza Awb-GLS',
        'gls_meta_box_callback',
        'shop_order',
        'normal',
        'low'
    );
}

// Callback
function gls_meta_box_callback( $order ){
	global $wpdb;

	session_start();
	$error_awb = isset($_SESSION['generate_awb']) ? $_SESSION['generate_awb'] : false;
	$success_awb = isset($_SESSION['success_awb']) ? $_SESSION['success_awb'] : false;
	session_destroy();

	/**
	* The Order data
	*/
	$order = wc_get_order($order->ID);
	$order_data = $order->get_data();
	$order_id = $order_data['id'];

	/**
	* select data from db
	* check if there was an awb alrady generated
	*/
	$sql = "SELECT * FROM glscarrier
		WHERE order_id = $order_id";

	$ord_det = $wpdb->get_results( $sql );
	if($ord_det){
		$ord_det = $ord_det[0];
		include_once(plugin_dir_path( __FILE__ ) . 'admin/partials/awb-display.php' );
	}else{
		include_once(plugin_dir_path( __FILE__ ) . 'admin/partials/form.php' );
	}
}

/**
 * Saving data
 */
add_action( 'save_post', 'gls_save_meta_box_data' );

function gls_save_meta_box_data( $order_id ) {

	if(isset($_POST['GenerateAwb'])){
		create_awb($order_id);
	}elseif(isset($_POST['regenAwb'])){
		create_awb($order_id);
		delete_awb($order_id);
	}elseif(isset($_POST['deleteAwb'])){
		delete_awb($order_id);
	}elseif(isset($_POST['UpdateZipOrder'])) {
		$order = wc_get_order($order_id);
		$order_data = $order->get_data();
		//update this address object
		if ($order->get_formatted_shipping_address()) {
			$order->set_shipping_postcode($_POST['zip_code_val']);
		} else {
			$order->set_billing_postcode($_POST['zip_code_val']);
		}

		$order->save();
	}
}

function get_order_details($order_id){
	global $wpdb;
	/**
	 * select data from db
	 * @var string
	 */
	$sql = "SELECT * FROM glscarrier
		WHERE order_id = $order_id";

	$ord_det = $wpdb->get_results( $sql );
	if($ord_det){
		$ord_det = $ord_det[0];
	}

	/**
	 * get consig details from order data
	 */
	$order = wc_get_order($order_id);
	$order_data = $order->get_data();

	if (isset($_POST['regenAwb']) && !empty($_POST['regenAwb']) || isset($_POST['deleteAwb']) && !empty($_POST['deleteAwb'])){
		$config = get_option($ord_det->cont_id);
	}else{
		$config = get_option($_POST['select_cont']);
	}

	if (isset($_POST['regenAwb']) && !empty($_POST['regenAwb'])){
		$cont_id = $ord_det->cont_id;
		$pcount = 1;
		$pickupdate = $ord_det->pickupdate;
		$content = $ord_det->content;
		$clientref = $ord_det->clientref;
		$codamount = $ord_det->codamount;
		$codref = $ord_det->codref; // info printed on label
		$services = json_decode($ord_det->services, True);
	}else{
		$codref = isset($_POST['codref']) && !empty($_POST['codref']) ? sanitize_text_field($_POST['codref']) : ''; // info printed on label
		$cont_id = isset($_POST['select_cont']) && !empty($_POST['select_cont']) ? sanitize_text_field($_POST['select_cont']) : '' ;
		$pcount = isset($_POST['pcount']) && !empty($_POST['pcount']) ? sanitize_text_field($_POST['pcount']) : '';
		$pickupdate = isset($_POST['pickupdate']) && !empty($_POST['pickupdate']) ? sanitize_text_field($_POST['pickupdate']) : '';
		$content = isset($_POST['content']) && !empty($_POST['content']) ? sanitize_text_field($_POST['content']) : '';
		$clientref = isset($_POST['clientref']) && !empty($_POST['clientref']) ? sanitize_text_field($_POST['clientref']) : '';
		$codamount = isset($_POST['codamount']) && !empty($_POST['codamount']) ? sanitize_text_field($_POST['codamount']) : '';
		$services = isset($_POST['service']) && !empty($_POST['service']) ? $_POST['service'] : '';
		if(isset($services[3]) && $services[3]['code'] == 'PRS'){
			$services[3]['info'] = $pickupdate;
			$data['printit'] = false;
		}
		if(isset($services[5]) && $services[5]['code'] == 'SM2'){
			$services[5]['info'] = $order_data['billing']['phone'];
		}
		if(isset($services[4]) && $services[4]['code'] == 'SM1' && isset($services[4]['info']) && !empty($services[4]['info'])){
			$services[4]['info'] = $order_data['billing']['phone'].'|'.$services[4]['info'];
		}
	}

	/**
	 * format phone number
	 */
	if (strpos($order_data['billing']['phone'], '+40') !== FALSE){
		$order_data['billing']['phone'] = $order_data['billing']['phone'];
	}else{
		$order_data['billing']['phone'] = '+4'.$order_data['billing']['phone'];
	}

	/**
	 * check if shipping name and last name is set and if not set consig name according with billing name and billing last name
	 */
	if($order->get_formatted_shipping_address()){
		$consig_name = $order_data['shipping']['first_name'].' '.$order_data['shipping']['last_name'];
	}else{
		$consig_name = $order_data['billing']['first_name'].' '.$order_data['billing']['last_name'];
	}

	/**
	 * check if shipping address is set  and if not set consig address according with billing address
	 */
	if($order->get_formatted_shipping_address()){
		$consig_address = $order_data['shipping']['address_1'];
	}else{
		$consig_address = $order_data['billing']['address_1'];
	}

	/**
	 * check if shipping city is set  and if not set consig city according with billing city
	 */
	if($order->get_formatted_shipping_address()){
		$consig_city = $order_data['shipping']['city'];
	}else{
		$consig_city = $order_data['billing']['city'];
	}

	/**
	 * check if shipping postcode is set  and if not set consig postcode according with billing postcode
	 */
	if($order->get_formatted_shipping_address()){
		$consig_postcode = $order_data['shipping']['postcode'];
	}else{
		$consig_postcode = $order_data['billing']['postcode'];
	}

	$ord = array(
		'consig_name'    => $consig_name,
		'consig_phone'   => $order_data['billing']['phone'],
		'consig_email'   => $order_data['billing']['email'],
		'consig_address' => $consig_address,
		'consig_city'    => $consig_city,
		'consig_zipcode' => $consig_postcode,
		'pcount'         => isset($pcount) ? $pcount : '',
		'content'        => isset($content) ? $content : '',
		'clientref' 	   => isset($clientref) ? $clientref : '',
		'codamount'      => isset($codamount) ? $codamount : '',
		'codref'         => isset($codref) ? $codref : '',
		'cont_id'        => isset($cont_id) ? $cont_id : '',
		'services'        => !empty($services) ? $services : '',
							);

	$data = array(
		'pickupdate' => isset($pickupdate) ? $pickupdate : '',
		'timestamp' => date('YmdHis',current_time('timestamp')),//'20140129150000',
		);


	// if (isset($_POST['regenAwb']) && !empty($_POST['regenAwb'])){
	// }
	$data = array_merge($config, $data);

	if(isset($services[3]) && $services[3]['code'] == 'PRS'){
		// $data['printit'] = false;
		$data['codamount'] = '';
	}

	if(isset($services[3]) && $services[3]['code'] == 'PRS' || isset($services[6]) && $services[6]['code'] == 'XS'){
		$data['sender_name'] = $consig_name;
		$data['sender_address'] = $consig_address;
		$data['sender_city'] = $consig_city;
		$data['sender_zipcode'] = $consig_postcode;
		$data['sender_country'] = 'RO';
		$data['sender_contact'] = $consig_name;
		$data['sender_phone'] = $order_data['billing']['phone'];
		$data['sender_email'] = $order_data['billing']['email'];
		$data['consig_name'] = $config['sender_name'];
		$data['consig_address'] = $config['sender_address'];
		$data['consig_city'] = $config['sender_city'];
		$data['consig_zipcode'] = $config['sender_zipcode'];
		$data['consig_country'] = 'RO';
		$data['consig_contact'] = $config['sender_contact'];
		$data['consig_phone'] = $config['sender_phone'];
		$data['consig_email'] = $config['sender_email'];
	}
	$data = array_merge($ord, $data);
	// echo '<pre>';
	// print_r($data);
	// echo '</pre>';
	// die();
	return $data;

}

function create_awb($order_id){
	global $wpdb;

	session_start();

	$ord_det = get_order_details($order_id);

	$gls_api = new Gls_api();
	$gls_api->initializate($order_id);

	$gls_api->setData($ord_det);

	$response = $gls_api->generateParcel();

	if(isset($response['successfull']) && $response['successfull'] == true){
		if(!empty($response['pcls'][0])){
      $parcelId = $response['pcls'][0];
      generate_pdf($response, $order_id);
    }else{
      $parcelId = '';
    }
		// save ship_parcel_id in db
    $wpdb->insert(
		    'glscarrier',
		    array(
		        'order_id'       => (int) $order_id,
		        'ship_parcel_id' => (int)$parcelId,
		        'pickupdate' 		 => $ord_det['pickupdate'],
		        'content'        => $ord_det['content'],
		        'clientref'      => $ord_det['clientref'],
		        'codamount'      => $ord_det['codamount'],
		        'codref'      	 => $ord_det['codref'],
		        'services'       => json_encode($ord_det['services']),
		        'date_add'       => date('Y-m-d H:i:s'),
		        'cont_id'        => $ord_det['cont_id'],
		    )
		);
		$_SESSION["success_awb"] = true;
	}else{
		$error_awb = '<div class="module_error alert alert-danger"><button type="button" class="close" data-dismiss="alert">×</button>'.error_response($response)["errdesc"].'</div>';

		$_SESSION["generate_awb"] = $error_awb;
	}
}

function delete_awb($order_id){
	global $wpdb;
	session_start();
	/**
	 * select data from db
	 * @var string
	 */
	$sql = "SELECT * FROM glscarrier
		WHERE order_id = $order_id";

	$generated_awb = $wpdb->get_results( $sql );
	if($generated_awb){
		$generated_awb = $generated_awb[0];
	}
	$ord_det = get_order_details($order_id);
	$_POST['select_cont'] = $generated_awb->cont_id;



	/**
	 * delete parcel
	 * @var Gls_api
	 */
	$gls_api = new Gls_api();
	$gls_api->initializate($order_id);
	$gls_api->setData($ord_det);
	$response = $gls_api->deleteParcel($generated_awb->ship_parcel_id);
  if(isset($response['successfull']) && $response['successfull'] == true){
		/**
		 * delete files
		 */
		$path = plugin_dir_path( dirname( __FILE__ ) ) .'gls/assets/awbs/';
		$files = [
				$path.$generated_awb->ship_parcel_id.'.pdf',
				$path.'ex_'.$generated_awb->ship_parcel_id.'.pdf'
			];
		foreach ($files as $file) {
			if (file_exists($file)) {
				unlink($file);
			}
		}

		/**
		 * delete entry from database
		 */
	  $wpdb->delete(
		    'glscarrier',
		    array(
		        'ship_parcel_id' => $generated_awb->ship_parcel_id,
		    )
		);
		$_SESSION["success_awb"] = true;
  }else{
		$error_awb = '<div class="module_error alert alert-danger"><button type="button" class="close" data-dismiss="alert">×</button>'.$response["errdesc"].'</div>';
		$_SESSION["generate_awb"] = $error_awb;
  }
}


function error_response($response){
		$all_responses = array(
			'1' =>'Autentificarea a eșuat',
			'2' =>'Hash nu este valid',
			'3' =>'Nu pot fi stocate date încercați din nou mai târziu',
			'4' =>'Șablonul de printare nu este valid. Vă rugăm să verificați documentația!',
			'5' =>'Parametri lipsa',
			'6' =>'Marcajul temporal nu este valid',
			'7' =>'Țara expeditorului nu este validă',
			'8' =>'Țara destinatarului nu este validă',
			'9' =>'Codul poștal al expeditorului nu este valid',
			'10' =>'Codul poștal al destinatarului nu este valid',
			'11' =>'Data de preluare nu este validă',
			'12' =>'Numarul de colete trebuie sa fie intre 1 and 99',
			'13' =>'Lipsa persoana de contact',
			'14' =>'Rambursul nu este permis in tara destinatarului',
			'15' =>'Valoarea maxima a rambursului este 7000 RON:',
			'17' =>'Cod serviciu gresit:',
			'18' =>'Combinațiile de servicii selectate nu sunt valide:',
			'19' =>'Serviciul (e) nu este disponibil în țara de preluare:',
			'20' =>'Serviciile nu sunt disponibile între țara expeditorului și destinatar:',
			'21' =>'Serviciul (e) nu este disponibil în țara / codul poștal al destinatarului:',
			'22' =>'Parametri lipsă pentru serviciul(e) selectat(e) (ex: mesaj sms, numar telefon, etc):',
			'23' =>'Serviciul FlexDeivery SMS este valabil numai pentru serviciul FlexDelivery',
			'25' =>'Aceeași solicitare trimisă de 5 ori în ultimele 5 minute!',
			'26' =>'AWB-ul solicitat nu apartine utilizatorului!',
			'27' =>'Format XML greșit!',
			'28' =>'Unul sau mai multe AWB-uri nu pot fi sterse.',
			'29' =>'Niciun AWB de sters.',
			'30' =>'AWB-ul a fost deja sters inainte de aceasta solicitare.',
			'31' =>'AWB-ul nu a fost gasit.',
		);
		$error = array($response);
		foreach ($error as $key => $value) {
			$response['errdesc'] = $all_responses[$value['errcode']];
		}

		return $response;

	}

	function generate_pdf($response, $order_id){
		if(isset($response['pdfdata']) && !empty($response['pdfdata'])){
			$ord_det = get_order_details($order_id);

			$path = plugin_dir_path( dirname( __FILE__ ) ) .'gls/assets';

			if(!file_exists($path)){
				mkdir($path);
			}
			$pdf_base64 = $path.'/test.txt';
			file_put_contents($pdf_base64, ''."\n", FILE_APPEND);
			//Get File content from txt file
			$pdf_base64_handler = fopen($pdf_base64,'r');
			$pdf_content = fread ($pdf_base64_handler,filesize($pdf_base64));
			fclose ($pdf_base64_handler);
			//Decode pdf content
			$pdf_decoded = base64_decode ($response['pdfdata']);
			//Write data back to pdf file
			if(!file_exists($path.'/awbs/')){
				mkdir($path.'/awbs/');
			}
			$pdf = fopen (plugin_dir_path( dirname( __FILE__ ) ) .'gls/assets/awbs/'.((int)$response['pcls'][0]).'.pdf','w');
			if (isset($ord_det['services'][6]) && $ord_det['services'][6]['code'] == 'XS') {
				$pdf_nd = fopen (plugin_dir_path( dirname( __FILE__ ) ) .'gls/assets/awbs/'.'ex_'.((int)$response['pcls'][0]).'.pdf','w');
				fwrite ($pdf_nd,$pdf_decoded);
					//close output file
				fclose ($pdf_nd);
			}
			fwrite ($pdf,$pdf_decoded);
			//close output file
			fclose ($pdf);
		}else{
			return false;
		}
	}

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-gls.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_gls() {

	$plugin = new Gls();
	$plugin->run();

}
run_gls();
